
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: ContribPlugin(BuildFileAddress(BuildFile(contrib/buildgen/src/python/pants/contrib/buildgen/core/BUILD, FileSystemProjectTree(/Users/mateo/dev/pants)), plugin))

from setuptools import setup

setup(**
{   'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: MacOS :: MacOS X',
                       'Operating System :: POSIX :: Linux',
                       'Programming Language :: Python',
                       'Topic :: Software Development :: Build Tools',
                       'Topic :: Software Development :: Code Generators'],
    'description': "Foursquare's Buildgen Pants plugin core module",
    'entry_points': {   'pantsbuild.plugin': [   'build_file_aliases = pants.contrib.buildgen.core.register:build_file_aliases',
                                                 'register_goals = pants.contrib.buildgen.core.register:register_goals']},
    'install_requires': ['ansicolors==1.0.2', 'pantsbuild.pants>=1.0.0,<1.5.0'],
    'license': 'Apache License, Version 2.0',
    'long_description': None,
    'name': 'fsqio.pants.contrib.buildgen.core',
    'namespace_packages': [   'pants',
                              'pants.contrib',
                              'pants.contrib.buildgen',
                              'pants.contrib.buildgen.core',
                              'pants.contrib.buildgen.core.subsystems'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   'pants',
                    'pants.contrib',
                    'pants.contrib.buildgen',
                    'pants.contrib.buildgen.core',
                    'pants.contrib.buildgen.core.subsystems'],
    'url': 'https://github.com/foursquare/fsqio',
    'version': '1.1.0',
    'zip_safe': True}
)
